using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace LD28_Warmup
{
    public class Game : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont font;

        public static int tileSize = 32;
        public static int screenWidth = 1024;
        public static int screenHeight = 640;
        public GameState state = GameState.AskFullScreen;
        public enum GameState
        {
            AskFullScreen,
            Playing,
            GameOver
        }
        private Snake snake;

        public Game()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Ridiculous folder";
        }


        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = screenWidth;
            graphics.PreferredBackBufferHeight = screenHeight;
            graphics.ApplyChanges();

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            font = Content.Load<SpriteFont>("font");
            snake = new Snake(GraphicsDevice, font);
        }

        protected override void UnloadContent()
        {
            // TODO: Nothing
        }

        protected override void Update(GameTime gameTime)
        {
            KeyboardState keyboard = Keyboard.GetState();

            if (keyboard.IsKeyDown(Keys.Escape)) this.Exit(); // Exit when pressing escape

            switch (state)
            {
                case GameState.AskFullScreen:
                    if (keyboard.IsKeyDown(Keys.Y))
                    {
                        toggleFullScreen();
                        state = GameState.Playing;
                    }
                    else if (keyboard.IsKeyDown(Keys.N))
                    {
                        state = GameState.Playing;
                    }
                    break;
                case GameState.Playing:
                    if (keyboard.IsKeyDown(Keys.F1)) toggleFullScreen(); // fullscreen when you need it!
                    snake.update(gameTime);
                    break;
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin();
            switch (state)
            {
                case GameState.AskFullScreen :
                    string q = "Do you want to enable fullscreen? (y / n)";
                    spriteBatch.DrawString(font, q, centerScreenDraw(q), Color.White);
                    break;
                case GameState.Playing :
                    snake.draw(spriteBatch);
                    break;
            }
            spriteBatch.End();

            base.Draw(gameTime);
        }

        public static Vector2 centerScreenDraw(string text)
        {
            int size = 14;
            int y = (screenHeight / 2) - (size / 2);
            int x = (screenWidth / 2) - ((text.Length * size) / 2);
            return new Vector2(x, y);
        }

        public void toggleFullScreen()
        {
            bool b = graphics.IsFullScreen;
            if (b == true) graphics.IsFullScreen = false;
            else graphics.IsFullScreen = true;
            graphics.ApplyChanges();
        }

        public static Texture2D emptyTex(int width, int height, Color color, GraphicsDevice device)
        {
            Texture2D tex = new Texture2D(device, width, height);
            Color[] clr = new Color[width * height];
            for (int i = 0; i < clr.Length; i++) clr[i] = color;
            tex.SetData(clr);
            return tex;
        }
    }
}
