﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace LD28_Warmup
{
    class Snake
    {
        public int realSpeed = 200;
        int speed;
        SnakePiece[] snake;
        int score = 0;
        int lastNull = 1;
        int elasped;
        Vector2 pos = new Vector2(20, 20);
        Texture2D tex;
        GraphicsDevice device;
        Food food;
        SpriteFont font;
        bool failed = false;

        Direction direction = Direction.Up;
        enum Direction
        {
            Up,
            Down,
            Left,
            Right
        }

        public Snake(GraphicsDevice dev, SpriteFont font) { init(dev, font); }

        private void init(GraphicsDevice dev, SpriteFont font)
        {
            this.tex = Game.emptyTex(Game.tileSize, Game.tileSize, Color.Red, dev);
            this.snake = new SnakePiece[(Game.screenHeight / Game.tileSize) * (Game.screenWidth / Game.tileSize)];
            this.score = 0;
            this.lastNull = 1;
            this.elasped = 0;
            this.device = dev;
            this.font = font;
            this.speed = realSpeed;
            this.failed = false;
            this.snake[0] = new SnakePiece(tex, new Vector2(Game.screenWidth / 2, Game.screenHeight / 2));
            this.food = new Food(Game.emptyTex(Game.tileSize, Game.tileSize, Color.Green, dev), snake);
        }

        public void update(GameTime gt)
        {
            KeyboardState state = Keyboard.GetState();
            if (!failed)
            {
                Vector2 prevPos = Vector2.Zero;
                elasped += gt.ElapsedGameTime.Milliseconds;
                bool makeMeNewPiecePlease = false;
                bool doneSomething = false;

                if (state.IsKeyDown(Keys.Up) && direction != Direction.Down) { direction = Direction.Up; doneSomething = true; }
                else if (state.IsKeyDown(Keys.Down) && direction != Direction.Up) { direction = Direction.Down; doneSomething = true; }
                else if (state.IsKeyDown(Keys.Left) && direction != Direction.Right) { direction = Direction.Left; doneSomething = true; }
                else if (state.IsKeyDown(Keys.Right) && direction != Direction.Left) { direction = Direction.Right; doneSomething = true; }

                if (state.IsKeyDown(Keys.PageUp)) speed--;
                else if (state.IsKeyDown(Keys.PageDown)) speed++;

                if (doneSomething) realSpeed = 50;
                else realSpeed = speed;

                if (elasped >= realSpeed)
                {
                    for (int i = 0; i < snake.Length; i++)
                    {
                        if (i == lastNull && makeMeNewPiecePlease)
                        {
                            snake[i] = new SnakePiece(tex, prevPos);
                        }

                        if (snake[i] == null)
                        {
                            lastNull = i;
                            break;
                        }
                        if (i == 0)
                        {
                            prevPos = snake[i].pos;

                            switch (direction)
                            {
                                case Direction.Up: snake[i].pos.Y -= Game.tileSize; break;
                                case Direction.Down: snake[i].pos.Y += Game.tileSize; break;
                                case Direction.Left: snake[i].pos.X -= Game.tileSize; break;
                                case Direction.Right: snake[i].pos.X += Game.tileSize; break;
                            }

                            if (snake[i].pos.X > Game.screenWidth - Game.tileSize) snake[i].pos.X = 0;
                            else if (snake[i].pos.X < 0) snake[i].pos.X = Game.screenWidth - Game.tileSize;
                            if (snake[i].pos.Y > Game.screenHeight - Game.tileSize) snake[i].pos.Y = 0;
                            else if (snake[i].pos.Y < 0) snake[i].pos.Y = Game.screenHeight - Game.tileSize;

                            if (snake[i].pos == food.pos)
                            {
                                makeMeNewPiecePlease = true;
                                score++;
                                food.newRandomPosition(snake);
                            }

                            bool hasFailed = false;
                            for (int a = 1; a < snake.Length; a++)
                            {
                                if (snake[a] == null) break;
                                if (snake[a].pos == snake[i].pos)
                                {
                                    hasFailed = true;
                                    failed = true;
                                    break;
                                }
                            }
                            if (hasFailed) break;
                        }
                        else
                        {
                            Vector2 pos = snake[i].pos;
                            snake[i].pos = prevPos;
                            prevPos = pos;
                        }                       
                    } // end for     
                    elasped = 0;
                }//end if
            }//end other if
            else
            {
                MouseState mouse = Mouse.GetState();
                if(mouse.LeftButton == ButtonState.Pressed || mouse.RightButton == ButtonState.Pressed) this.init(device, font);
            }
        }

        public void draw(SpriteBatch sb)
        {
            for (int i = 0; i < snake.Length; i++)
            {
                if (snake[i] == null) break;
                snake[i].draw(sb);
            }
            food.draw(sb);

            if (!failed) sb.DrawString(font, "Score: " + score, pos, Color.White);
            else
            {
                sb.DrawString(font, "You failed! Score: " + score, Game.centerScreenDraw("LOL LOL LOL LOL LOL" + score.ToString()), Color.White);
                sb.DrawString(font, "Click to continue!", new Vector2(Game.centerScreenDraw("AAAAAAAAAAAAAAAA").X, (Game.screenHeight / 2) + 20), Color.White); // messed up
            }

        }
    }

    class SnakePiece
    {
        public Texture2D tex;
        public Vector2 pos;

        public SnakePiece(Texture2D texture, Vector2 position)
        {
            this.tex = texture;
            this.pos = position;
        }

        public void draw(SpriteBatch sb)
        {
            sb.Draw(tex, pos, Color.White);
        }
    }
}
